/* -----------------------------------------------------------------------------
 *
 *	crosser~
 *		creates zero-crossing overshoot error
 *		y[n] = ( x[n] + ( x[n] * depth )) - depth
 *
 *	2007 . Cooper Baker . www.cooperbaker.com
 *
 *----------------------------------------------------------------------------*/

#include <Carbon/Carbon.h>
#include <MaxAPI/MaxAPI.h>
#include <MaxAudioAPI/MaxAudioAPI.h>

void *crosser_class;									// pointer to this 'class'

// crosser~ structure ------------------------------------------------------ 
typedef struct _crosser									// crosser data structure
{ 
    t_pxobject x_obj;									// every external needs this; t_pxobject struct contains object info
} t_crosser; 

// prototypes ------------------------------------------------------------------ 
void *crosser_new(void);								// makes a new crosser~ object
void crosser_free(t_crosser *x);						// deletes the crosser~ object
void crosser_dsp(t_crosser *x, t_signal **sp, short *count); // sets up dsp for crosser
t_int *crosser_perform(t_int *w);						// the dsp perform function
void crosser_assist(t_crosser *x, void *b, long m, long a, char *s);
														// inlet / outlet assistance
// main ------------------------------------------------------------------------
void main(void) 
{ 
	setup((t_messlist **)&crosser_class, (method)crosser_new, (method)crosser_free, (short)sizeof(t_crosser), 0L, 0); 
	addmess((method)crosser_dsp, "dsp", A_CANT, 0);		// add dsp method
	addmess((method)crosser_assist, "assist", A_CANT, 0);// assist method 
	dsp_initclass();									// init the class
	post( "" );
	post( "cb.crosser~ :: creates zero-crossing overshoot error" );
	post( "2007 . Cooper Baker . www.cooperbaker.com" );
	post( "" );
}														// post after init completes

// crosser_dsp --------------------------------------------------------------------
void crosser_dsp(t_crosser *x, t_signal **sp, short *count)	// crosser_dsp(*data struct, **signal struct, signal count )
{ 
	dsp_add( crosser_perform, 5, sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n, x );
}						   // 5 = number of values passed to crosser_perform

// crosser_perform ---------------------------------------------------------------- 
t_int *crosser_perform(t_int *w)						// crosser_perform runs once per signal vector
{ 
	t_float *in1	= (t_float *)(w[1]);				// left inlet signal vector location
	t_float *in2	= (t_float *)(w[2]);				// right inlet signal vector location
	t_float *out	= (t_float *)(w[3]);				// outlet signal vector location
	int n			= (int)(w[4]);						// vector size in samples
	t_crosser *x	= (t_crosser *)(w[5]);				// crosser struct location
	float sample	= 0;								// the current sample
	float depth		= 0;								// depth of bad zero cross
	
// DSP BLOCK ///////////////////////////////////////////////////////////////////	
	while ( n-- )										// while a sample remains, do dsp ( n-- is the counter )
	{  
		sample	= *in1++;								// *in++/*out++ should only be done once per while loop cycle
		depth	= *in2++;								// *in++/*out++ should only be done once per while loop cycle

		if( sample > 0 )								// if sample is above zero
		{												// overshoot zero	
			sample = ( sample + ( sample * depth )) - depth;
			goto exit;									// exit
		}
		if( sample < 0 )								// if sample is below zero
		{												// overshoot zero
			sample = ( sample + ( sample * depth )) + depth;
			goto exit;									// exit
		}
		
		exit :											// the exit
		*out++ = sample;								// sample is used to make things clearer
	}
// END DSP BLOCK ///////////////////////////////////////////////////////////////	
	
	return (w + 6);										// always return w plus one more than the 2nd argument in dsp_add() 
} 

// crosser_assist -----------------------------------------------------------------
void crosser_assist(t_crosser *x, void *b, long m, long a, char *s)
{
	switch (m)											// inlet / outlet switch
	{
		case 1:											// 1 = inlets
			switch (a)									// inlet number switch
			{
				case 0:									// inlet 0
					sprintf(s, "(signal) crosser~ input");
					break;								// assist message
				case 1:									// inlet 1
					sprintf(s, "(signal) crosser~ depth");
					break;								// assist message
				default:
					break;
			}
			break;
		case 2:											// 2 = outlets
			switch (a)									// outlet number switch
			{
				case 0:									// outlet 0
					sprintf(s, "(signal) crosser~ output");
					break;								// assist message
				default:
					break;
			}
			break;
	}
}

// crosser_new() ------------------------------------------------------------------ 
void *crosser_new(void)									// crosser~ 'constructor'
{ 
	t_crosser *x = (t_crosser *)newobject(crosser_class); // register the object with crosser and save its address as *x
	dsp_setup((t_pxobject *)x, 2);						// create inlets
	outlet_new((t_pxobject *)x, "signal");				// create outlet 
	return (x);											// where am i?
} 

// crosser_free() -----------------------------------------------------------------
void crosser_free(t_crosser *x)							// crosser~ 'destructor'
{
	dsp_free((t_pxobject *)x);							// crosser free function
}
