/* -----------------------------------------------------------------------------
 *
 *	cb.oversat~
 *		simulates analog saturation above a variable threshold
 *
 *		y[n] = ((((x[n] - t ) * m ) * x[n] ) + t ) * ( 1 / ((( 1- t ) * m ) + t ));
 *		t = threshold, m = multiplier
 *		
 *		2007 . Cooper Baker . www.cooperbaker.com
 *
 *----------------------------------------------------------------------------*/

#include <Carbon/Carbon.h>
#include <MaxAPI/MaxAPI.h>
#include <MaxAudioAPI/MaxAudioAPI.h>

void *oversat_class;										// pointer to this 'class'

// oversat~ structure ------------------------------------------------------ 
typedef struct _oversat										// oversat data structure
{ 
	t_pxobject	x_obj;									// every oversat external needs this; t_pxobject struct contains object info
	float		samp;
	float		thr_pos;
	float		thr_neg;
	float		mult;
} t_oversat; 

// prototypes ------------------------------------------------------------------ 
void *oversat_new(void);									// makes a new oversat~ object
void oversat_free(t_oversat *x);								// deletes the oversat~ object
void oversat_dsp(t_oversat *x, t_signal **sp, short *count);	// sets up dsp for oversat
t_int *oversat_perform(t_int *w);							// the dsp perform function
void oversat_assist(t_oversat *x, void *b, long m, long a, char *s);
														// inlet / outlet assistance
// main ------------------------------------------------------------------------
void main(void) 
{ 
	setup((t_messlist **)&oversat_class, (method)oversat_new, (method)oversat_free, (short)sizeof(t_oversat), 0L, 0); 
	addmess((method)oversat_dsp, "dsp", A_CANT, 0);			// add dsp method
	addmess((method)oversat_assist, "assist", A_CANT, 0);	// assist method 
	dsp_initclass();									// init the class
	post( "" );								
	post( "cb.oversat~ :: simulates oversaturation above a variable threshold" );
	post( "2007 . Cooper Baker . www.cooperbaker.com" );
	post( "" );
}														// post after init completes

// oversat_dsp --------------------------------------------------------------------
void oversat_dsp(t_oversat *x, t_signal **sp, short *count)		// oversat_dsp(*data struct, **signal struct, signal count )
{														// add oversat_perform to the dsp chain with 4 values
	dsp_add( oversat_perform, 6, sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[3]->s_vec, sp[0]->s_n, x );
}						   // 6 = number of values passed to oversat_perform

// oversat_perform ---------------------------------------------------------------- 
t_int *oversat_perform(t_int *w)						// oversat_perform runs once per signal vector
{ 
	t_float		*in1	= (t_float *)(w[1]);			// inlet signal vector location
	t_float		*in2	= (t_float *)(w[2]);			// inlet signal vector location
	t_float		*in3	= (t_float *)(w[3]);			// inlet signal vector location
	t_float		*out	= (t_float *)(w[4]);			// outlet signal vector location
	int			n		= (int)(w[5]);					// vector size in samples
	t_oversat	*x		= (t_oversat *)(w[6]);			// oversat struct location

// DSP BLOCK ///////////////////////////////////////////////////////////////////	
	while ( n-- )										// while a sample remains, do dsp ( n-- is the counter )
	{  
		x->samp		= *in1++;									// *in++/*out++ should only be done once per while loop cycle
		x->thr_pos	= *in2++;
		x->mult		= *in3++;
		
		x->thr_pos = ( x->thr_pos >= 0 ) ? x->thr_pos : 0;
		x->thr_neg = x->thr_pos * -1;
		
		if( x->samp > x->thr_pos)
		{
			x->samp = ((((x->samp-x->thr_pos)*x->mult)*x->samp)+x->thr_pos)*(1/(((1-x->thr_pos)*x->mult)+x->thr_pos));
		}
		if( x->samp < x->thr_neg )
		{
			x->samp = ((((x->thr_neg-x->samp)*x->mult)*x->samp)+x->thr_neg)*(1/(((1-x->thr_pos)*x->mult)+x->thr_pos));
		}
		
		*out++ = x->samp;
	}
// END DSP BLOCK ///////////////////////////////////////////////////////////////	
	
	return (w + 7);										// always return w plus one more than the 2nd argument in dsp_add() 
} 

// oversat_assist -----------------------------------------------------------------
void oversat_assist(t_oversat *x, void *b, long m, long a, char *s)
{
	switch (m)											// inlet / outlet switch
	{
		case 1:											// 1 = inlets
			switch (a)									// inlet number switch
			{
				case 0:									// inlet 0
					sprintf(s, "(signal) cb.oversat~ Input");
					break;								// assist message
				case 1:									// inlet 0
					sprintf(s, "(signal) cb.oversat~ Threshold");
					break;								// assist message
				case 2:									// inlet 0
					sprintf(s, "(signal) cb.oversat~ Multiplier");
					break;								// assist message
				default:
					break;
			}
			break;
		case 2:											// 2 = outlets
			switch (a)									// outlet number switch
			{
				case 0:									// outlet 0
					sprintf(s, "(signal) cb.oversat~ Output");
					break;								// assist message
				default:
					break;
			}
			break;
	}
}

// oversat_new() ------------------------------------------------------------------ 
void *oversat_new(void)										// oversat~ 'constructor'
{ 
	t_oversat *x = (t_oversat *)newobject(oversat_class);			// register the object with oversat and save its address as *x
	dsp_setup((t_pxobject *)x, 3);						// create inlets
	outlet_new((t_pxobject *)x, "signal");				// create left outlet 
	return (x);											// where am i?
} 

// oversat_free() -----------------------------------------------------------------
void oversat_free(t_oversat *x)									// oversat~ 'destructor'
{
	dsp_free((t_pxobject *)x);							// oversat free function
}
