/* -----------------------------------------------------------------------------
 *
 *	threshdrop~
 *		causes a single-sample drop-out at each wave peak above the threshold
 *		
 *
 *	2007 . Cooper Baker . www.cooperbaker.com
 *
 *----------------------------------------------------------------------------*/

#include <Carbon/Carbon.h>
#include <MaxAPI/MaxAPI.h>
#include <MaxAudioAPI/MaxAudioAPI.h>

void *threshdrop_class;										// pointer to this 'class'

// threshdrop~ structure ------------------------------------------------------ 
typedef struct _threshdrop									// threshdrop data structure
{ 
    t_pxobject x_obj;										// t_pxobject struct contains object info
	Float32		sample[3];									// sample memory for analysis
	Float32		drop;										// output sample	
	Float32		thresh;										// positive threshold
	Float32		negthresh;									// negative threshold
} t_threshdrop; 

// prototypes ------------------------------------------------------------------ 
void *threshdrop_new(void);									// makes a new threshdrop~ object
void threshdrop_free(t_threshdrop *x);						// deletes the threshdrop~ object
void threshdrop_dsp(t_threshdrop *x, t_signal **sp, short *count); // sets up dsp for threshdrop
t_int *threshdrop_perform(t_int *w);						// the dsp perform function
void threshdrop_assist(t_threshdrop *x, void *b, long m, long a, char *s);
															// inlet / outlet assistance
// main ------------------------------------------------------------------------
void main(void) 
{ 
	setup((t_messlist **)&threshdrop_class, (method)threshdrop_new, (method)threshdrop_free, (short)sizeof(t_threshdrop), 0L, 0); 
	addmess((method)threshdrop_dsp, "dsp", A_CANT, 0);		// add dsp method
	addmess((method)threshdrop_assist, "assist", A_CANT, 0);// assist method 
	dsp_initclass();	
	post( "" );								
	post( "cb.threshdrop~ :: creates droupouts at peaks above a variable threshold" );
	post( "2007 . Cooper Baker . www.cooperbaker.com" );
	post( "" );
}															// post after init completes

// threshdrop_dsp --------------------------------------------------------------------
void threshdrop_dsp(t_threshdrop *x, t_signal **sp, short *count)
{
	dsp_add( threshdrop_perform, 5, sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n, x );
}							  // 5 = number of values passed to threshdrop_perform

// threshdrop_perform ---------------------------------------------------------------- 
t_int *threshdrop_perform(t_int *w)							
{ 
	t_float *in1	= (t_float *)(w[1]);					// inlet signal vector address
	t_float *in2	= (t_float *)(w[2]);					// inlet signal vector address
	t_float *out	= (t_float *)(w[3]);					// outlet signal vector address
	int n			= (int)(w[4]);							// vector size in samples
	t_threshdrop *x	= (t_threshdrop *)(w[5]);				// threshdrop struct address
	
	int i;
// DSP BLOCK ///////////////////////////////////////////////////////////////////	
	while ( n-- )											// while a sample remains, do dsp ( n-- is the counter )
	{  
		x->sample[2]	= x->sample[1];						// bucket-brigade the sample memory
		x->sample[1]	= x->sample[0];						// . . .
		x->sample[0]	= *in1++;							// *in++/*out++ should only be done once per while loop cycle
		x->drop			= x->sample[1];						// output middle sample[n]
		x->thresh		= *in2++;							// the threshold
		
		x->thresh = ( x->thresh < 0 ) ? x->thresh * -1 : x->thresh;
		// x->thresh = ( x->thresh > 1 ) ? 1 : x->thresh;	// fix negative input
		x->negthresh = x->thresh * -1;						// make negative threshold
		
		if( x->sample[1] > x->thresh )						// if above positive threshold
		{
			if( ( x->sample[0] < x->sample[1] ) && ( x->sample[1] > x->sample[2] ) )
			{												// if sample[1] is a peak
				x->drop = 0;								// drop it
			}
		}
		if( x->sample[1] < x->negthresh )					// if below neg threshold
		{
			if( ( x->sample[0] > x->sample[1] ) && ( x->sample[1] < x->sample[2] ) )
			{												// if sample[1] is a trough
				x->drop = 0;								// drop it
			}
		}
		*out++ = x->drop;									// copy sample to output vector and increment address
	}
// END DSP BLOCK ///////////////////////////////////////////////////////////////	
	
	return (w + 6);											// always return w plus one more than the 2nd argument in dsp_add() 
} 

// threshdrop_assist -----------------------------------------------------------------
void threshdrop_assist(t_threshdrop *x, void *b, long m, long a, char *s)
{
	switch (m)												// inlet / outlet switch
	{
		case 1:												// 1 = inlets
			switch (a)										// inlet number switch
			{
				case 0:										// inlet 0
					sprintf(s, "(signal) threshdrop~ input");
					break;									// assist message
				case 1:										// inlet 1
					sprintf(s, "(signal) threshdrop~ threshold");
					break;									// assist message
				default:
					break;
			}
			break;
		case 2:												// 2 = outlets
			switch (a)										// outlet number switch
			{
				case 0:										// outlet 0
					sprintf(s, "(signal) threshdrop~ output");
					break;									// assist message
				default:
					break;
			}
			break;
	}
}

// threshdrop_new() ------------------------------------------------------------------ 
void *threshdrop_new(void)									// threshdrop~ 'constructor'
{ 
	t_threshdrop *x = (t_threshdrop *)newobject(threshdrop_class);	// register the object with msp and save its address as *x
	dsp_setup((t_pxobject *)x, 2);							// create inlets 
	outlet_new((t_pxobject *)x, "signal");					// create outlet 
	
	x->sample[0] = 0;										// init x->sample array
	x->sample[1] = 0;										// . . .
	x->sample[2] = 0;										//
	
	return (x);												// where am i?
} 

// threshdrop_free() -----------------------------------------------------------------
void threshdrop_free(t_threshdrop *x)						// threshdrop~ 'destructor'
{
	dsp_free((t_pxobject *)x);								// mspp free function
}
