/* -----------------------------------------------------------------------------
 *
 *	threshflip~
 *		inverts signals above a complimentary positive/negative threshold
 *		y[n] = t + ( t - x[n] )
 *
 *	2007 . Cooper Baker . www.cooperbaker.com
 *
 *----------------------------------------------------------------------------*/

#include <Carbon/Carbon.h>
#include <MaxAPI/MaxAPI.h>
#include <MaxAudioAPI/MaxAudioAPI.h>

void *threshflip_class;									// pointer to this 'class'

// threshflip~ structure ------------------------------------------------------ 
typedef struct _threshflip								// threshflip data structure
{ 
    t_pxobject x_obj;									// every external needs this; t_pxobject struct contains object info
} t_threshflip; 

// prototypes ------------------------------------------------------------------ 
void *threshflip_new(void);								// makes a new threshflip~ object
void threshflip_free(t_threshflip *x);					// deletes the threshflip~ object
void threshflip_dsp(t_threshflip *x, t_signal **sp, short *count);	// sets up dsp for threshflip
t_int *threshflip_perform(t_int *w);					// the dsp perform function
void threshflip_assist(t_threshflip *x, void *b, long m, long a, char *s);
														// inlet / outlet assistance
// main ------------------------------------------------------------------------
void main(void) 
{ 
	setup((t_messlist **)&threshflip_class, (method)threshflip_new, (method)threshflip_free, (short)sizeof(t_threshflip), 0L, 0); 
	addmess((method)threshflip_dsp, "dsp", A_CANT, 0);	// add dsp method
	addmess((method)threshflip_assist, "assist", A_CANT, 0);// assist method 
	dsp_initclass();									// init the class
	post( "" );
	post( "cb.threshflip~ :: mirrors signals above variable threshold" );
	post( "2007 . Cooper Baker . www.cooperbaker.com" );
	post( "" );
}														// post after init completes

// threshflip_dsp --------------------------------------------------------------------
void threshflip_dsp(t_threshflip *x, t_signal **sp, short *count)	// threshflip_dsp(*data struct, **signal struct, signal count )
{ 
	dsp_add( threshflip_perform, 5, sp[0]->s_vec, sp[1]->s_vec, sp[2]->s_vec, sp[0]->s_n, x );
}							  // 5 = number of values passed to threshflip_perform

// threshflip_perform ---------------------------------------------------------------- 
t_int *threshflip_perform(t_int *w)						// threshflip_perform runs once per signal vector
{ 
	t_float *in1	= (t_float *)(w[1]);				// left inlet signal vector location
	t_float *in2	= (t_float *)(w[2]);				// right inlet signal vector location
	t_float *out	= (t_float *)(w[3]);				// outlet signal vector location
	int n			= (int)(w[4]);						// vector size in samples
	t_threshflip *x	= (t_threshflip *)(w[5]);			// threshflip struct location
	Float32	sample	= 0;								// the current sample
	Float32 pos		= 0;								// the positive threshold
	Float32	neg		= 0;								// the negative threshold
	
// DSP BLOCK ///////////////////////////////////////////////////////////////////	
	while ( n-- )										// while a sample remains, do dsp ( n-- is the counter )
	{  
		sample	= *in1++;								// *in++/*out++ should only be done once per while loop cycle
		pos		= *in2++;								// *in++/*out++ should only be done once per while loop cycle
		pos = ( pos < 0 ) ? 0 : pos;					// clip threshold below 0
		pos = ( pos > 1 ) ? 1 : pos;					// clip threshold above 1
		neg		= pos * -1;								// calculate negative thresh

		if( sample > pos )								// if sample is above pos thresh
		{	
			sample = pos + ( pos - sample );			// flip sample
			goto exit;									// exit
		}
		if( sample < neg )								// if sample is below neg thresh
		{
			sample = neg + ( neg - sample );			// flip sample
			goto exit;									// exit
		}
		
		exit :											// the exit
		*out++ = sample;								// sample is used to make things clearer
	}
// END DSP BLOCK ///////////////////////////////////////////////////////////////	
	
	return (w + 6);										// always return w plus one more than the 2nd argument in dsp_add() 
} 

// threshflip_assist -----------------------------------------------------------------
void threshflip_assist(t_threshflip *x, void *b, long m, long a, char *s)
{
	switch (m)											// inlet / outlet switch
	{
		case 1:											// 1 = inlets
			switch (a)									// inlet number switch
			{
				case 0:									// inlet 0
					sprintf(s, "(signal) threshflip~ input");
					break;								// assist message
				case 1:									// inlet 1
					sprintf(s, "(signal) threshflip~ threshold");
					break;								// assist message
				default:
					break;
			}
			break;
		case 2:											// 2 = outlets
			switch (a)									// outlet number switch
			{
				case 0:									// outlet 0
					sprintf(s, "(signal) threshflip~ output");
					break;								// assist message
				default:
					break;
			}
			break;
	}
}

// threshflip_new() ------------------------------------------------------------------ 
void *threshflip_new(void)								// threshflip~ 'constructor'
{ 
	t_threshflip *x = (t_threshflip *)newobject(threshflip_class);		// register the object with threshflip and save its address as *x
	dsp_setup((t_pxobject *)x, 2);						// create inlets
	outlet_new((t_pxobject *)x, "signal");				// create outlet 
	return (x);											// where am i?
} 

// threshflip_free() -----------------------------------------------------------------
void threshflip_free(t_threshflip *x)					// threshflip~ 'destructor'
{
	dsp_free((t_pxobject *)x);							// threshflip free function
}
